# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2009- Spyder Project Contributors and others (see LICENSE.txt)
#
# Licensed under the terms of the MIT and other licenses where noted
# (see LICENSE.txt in this directory and NOTICE.txt in the root for details)
# -----------------------------------------------------------------------------

"""
spyder.plugins.help.utils
=================

Configuration files for the Help plugin rich text mode.

See their headers, LICENSE.txt in this directory or NOTICE.txt for licenses.
"""

import sys
from spyder.config.base import get_module_source_path
sys.path.insert(0, get_module_source_path(__name__))
